# -*- coding: utf-8 -*-

# Copyright (C) 2010-2011  Dustin Widmann <1m.0n.f1r3@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import *
from PyKDE4.plasma import Plasma
from PyKDE4 import plasmascript
import statusimages
from dialog import *

class FapStatus(plasmascript.Applet):
    def __init__(self,parent,args=None):
        plasmascript.Applet.__init__(self,parent)

    def init(self):
        self.setHasConfigurationInterface(False)
        self.setAspectRatioMode(Plasma.Square)
        self.theme = Plasma.Svg(self)
        self.theme.setImagePath("widgets/background")
        self.setBackgroundHints(Plasma.Applet.DefaultBackground)
        self.greenIcon = QIcon(QPixmap(":/colors/green.png"))
        self.redIcon = QIcon(QPixmap(":/colors/red.png"))
        self.yellowIcon = QIcon(QPixmap(":/colors/yellow.png"))
        self.orangeIcon = QIcon(QPixmap(":/colors/orange.png"))
        self.grayIcon = QIcon(QPixmap(":/colors/gray.png"))
        self.green12Icon = QIcon(QPixmap(":/colors/green12.png"))
        self.green25Icon = QIcon(QPixmap(":/colors/green25.png"))
        self.green37Icon = QIcon(QPixmap(":/colors/green37.png"))
        self.green50Icon = QIcon(QPixmap(":/colors/green50.png"))
        self.green62Icon = QIcon(QPixmap(":/colors/green62.png"))
        self.green75Icon = QIcon(QPixmap(":/colors/green75.png"))
        self.green87Icon = QIcon(QPixmap(":/colors/green87.png"))
        self.pearlIcon = QIcon(QPixmap(":/colors/pearl.png"))

        self.currentIconWidget = Plasma.IconWidget()
        self.currentIconWidget.setIcon(self.grayIcon)
        #self.currentIconWidget.setToolTip("Applet now started.")

        self.currentPercentWidget = Plasma.Label()
        self.currentPercentWidget.setText('n/a')
        self.currentPercentWidget.setAlignment(Qt.AlignHCenter) #Horizontally centered
        self.layout = QGraphicsLinearLayout(Qt.Vertical, self.applet)
        self.layout.addItem(self.currentIconWidget)
        self.layout.addItem(self.currentPercentWidget)
        self.setLayout(self.layout)

        self.Dialog = PopupDialog()
        self.Dialog.init()
        self.dialogTimer = QTimer()
        self.Dialog.hide()

        self.setUserMessage("FAP Status","Applet Started")
        #self.connect(self.dialogTimer, SIGNAL("timeout()"), self.showDialog)
        self.connect(self.dialogTimer, SIGNAL("timeout()"), self.hideDialog)
        self.connect(self.currentIconWidget, SIGNAL("hoverEnterEvent()"), self.hoverEnterEvent)
        self.connect(self.currentIconWidget, SIGNAL("hoverLeaveEvent()"), self.hoverLeaveEvent)

        #self.resize(30, 30)
        self.connectToEngine()
        self.update()

    def setUserMessage(self, s1, s2=""):
        # Update the hover dialog to give user information
        self.Dialog.setTitle(s1)
        self.Dialog.setBody(s2)

    def hoverEnterEvent(self, event):
        self.dialogTimer.start(5000)
        self.showDialog()

    def showDialog(self):
        self.Dialog.move(self.popupPosition(self.Dialog.sizeHint()))
        self.Dialog.showDialog()

    def hideDialog(self):
        self.Dialog.hide()
    def hoverLeaveEvent(self, event):
        self.dialogTimer.stop()
        self.Dialog.hide()

    def connectToEngine(self):
        self.FapStatusEngine = self.dataEngine("engine-fap-monitor")
        self.FapStatusEngine.connectSource("HN9000",self,1000)
		
    @pyqtSignature("dataUpdated(const QString &, const Plasma::DataEngine::Data &)")
    def dataUpdated(self, sourceName, data):
        print(str(data))
        self.current_status = str(data[QString("hn9_current_status")])
        self.download_allowance = str(data[QString("hn9_download_allowance")])
        self.download_remaining = str(data[QString("hn9_download_remaining")])
        self.refill = str(data[QString("hn9_refill")])

        status_message = {}
        status_message['gray'] = "Could not reach modem"
        status_message['green'] = "OK"
        status_message['red'] = "Down"
        status_message['orange'] = "Fapped"
        status_message['yellow'] = "Degraded"
        status_message['pearl'] = 'Free time'

        if self.current_status == "red":
            self.currentIconWidget.setIcon(self.redIcon)
        elif self.current_status == "green":
            percent = int(float(self.download_remaining) / float(self.download_allowance) * 100.0)
            print(percent)
            if percent > 93:
                self.currentIconWidget.setIcon(self.greenIcon)
            elif percent >= 81 and percent < 93:
                self.currentIconWidget.setIcon(self.green87Icon)
            elif percent >= 69 and percent < 81:
                self.currentIconWidget.setIcon(self.green75Icon)
            elif percent >= 56 and percent < 69:
                self.currentIconWidget.setIcon(self.green62Icon)
            elif percent >= 44 and percent < 56:
                self.currentIconWidget.setIcon(self.green50Icon)
            elif percent >= 31 and percent < 44:
                self.currentIconWidget.setIcon(self.green37Icon)
            elif percent >=19 and percent < 31:
                self.currentIconWidget.setIcon(self.green25Icon)
            elif percent >= 6.25 and percent < 19:
                self.currentIconWidget.setIcon(self.green12Icon)
            else:
                self.currentIconWidget.setIcon(self.grayIcon)
        elif self.current_status == "yellow":
            self.currentIconWidget.setIcon(self.yellowIcon)
        elif self.current_status == "orange":
            self.currentIconWidget.setIcon(self.orangeIcon)
            self.downloadRemaining = 0
            
        else:
            self.currentIconWidget.setIcon(self.grayIcon)
        self.currentPercentWidget.setText(str(percent) + "%")
        from datetime import datetime
        from pytz import timezone
        import pytz
        utc = pytz.utc
        eastern = timezone('US/Eastern')

        naive_time = datetime.utcnow()
        utc_time = utc.localize(datetime(naive_time.year,naive_time.month,naive_time.day,naive_time.hour,naive_time.minute,naive_time.second))
        eastern_time = utc_time.astimezone(eastern)

        if 2 <= eastern_time.hour < 7:
            self.currentIconWidget.setIcon(self.pearlIcon)
            self.current_status = 'pearl'
        print("Eastern time:" +str(eastern_time.hour))
        print(self.current_status)
        self.setUserMessage("Hughesnet Status","Status: %s\nPlan Allowance: %s\nPlan Remaining: %s\nNext Refill: %s" % (status_message[self.current_status],self.download_allowance,self.download_remaining,self.refill))
        self.update()
def CreateApplet(parent):
    return FapStatus(parent)
